@@ -11,10 +11,11 @@ from collections import defaultdict
 from xml.dom import SyntaxErr
 
 from lxml import etree
-import cssutils
-from cssutils.css import Property
+import css_parser
+from css_parser.css import Property
 
 from calibre import guess_type
+from calibre.ebooks import unit_convert
 from calibre.ebooks.oeb.base import (XHTML, XHTML_NS, CSS_MIME, OEB_STYLES,
         namespace, barename, XPath)
 from calibre.ebooks.oeb.stylizer import Stylizer
@@ -138,7 +139,7 @@ class EmbedFontsCSSRules(object):
             iid, href = oeb.manifest.generate(u'page_styles', u'page_styles.css')
             rules = [x.cssText for x in self.rules]
             rules = u'\n\n'.join(rules)
-            sheet = cssutils.parseString(rules, validate=False)
+            sheet = css_parser.parseString(rules, validate=False)
             self.href = oeb.manifest.add(iid, href, guess_type(href)[0],
                     data=sheet).href
         return self.href
@@ -179,6 +180,11 @@ class CSSFlattener(object):
             titlepage = titlepage.item
             if titlepage is not None and titlepage not in self.items:
                 self.items.append(titlepage)
+        epub3_nav = None
+        if getattr(self.opts, 'epub3_nav_href', None):
+            epub3_nav = self.oeb.manifest.hrefs.get(self.opts.epub3_nav_href)
+            if epub3_nav is not None and epub3_nav not in self.items:
+                self.items.append(epub3_nav)
 
         self.filter_css = frozenset()
         if self.opts.filter_css:
@@ -197,7 +203,7 @@ class CSSFlattener(object):
             # Make all links to resources absolute, as these sheets will be
             # consolidated into a single stylesheet at the root of the document
             if item.media_type in OEB_STYLES:
-                cssutils.replaceUrls(item.data, item.abshref,
+                css_parser.replaceUrls(item.data, item.abshref,
                         ignoreImportRules=True)
 
         self.body_font_family, self.embed_font_rules = self.get_embed_font_info(
@@ -210,6 +216,21 @@ class CSSFlattener(object):
         self.sbase = self.baseline_spine() if self.fbase else None
         self.fmap = FontMapper(self.sbase, self.fbase, self.fkey)
         self.flatten_spine()
+        if epub3_nav is not None:
+            self.opts.epub3_nav_parsed = epub3_nav.data
+
+        self.store_page_margins()
+
+    def store_page_margins(self):
+        self.opts._stored_page_margins = {}
+        for item, stylizer in self.stylizers.iteritems():
+            margins = self.opts._stored_page_margins[item.href] = {}
+            for prop, val in stylizer.page_rule.items():
+                p, w = prop.partition('-')[::2]
+                if p == 'margin':
+                    margins[w] = unit_convert(
+                            val, stylizer.profile.width_pts, stylizer.body_font_size,
+                            stylizer.profile.dpi, body_font_size=stylizer.body_font_size)
 
     def get_embed_font_info(self, family, failure_critical=True):
         efi = []
@@ -257,7 +278,7 @@ class CSSFlattener(object):
                     cfont[k] = font[k]
             rule = '@font-face { %s }'%('; '.join(u'%s:%s'%(k, v) for k, v in
                 cfont.iteritems()))
-            rule = cssutils.parseString(rule)
+            rule = css_parser.parseString(rule)
             efi.append(rule)
 
         return body_font_family, efi
@@ -436,12 +457,11 @@ class CSSFlattener(object):
             cssdict['font-weight'] = 'normal'  # ADE chokes on font-weight medium
 
         fsize = font_size
-        is_drop_cap = (cssdict.get('float', None) == 'left' and 'font-size' in
-                       cssdict and len(node) == 0 and node.text and
-                       (len(node.text) == 1 or (len(node.text) == 2 and 0x2000 <= ord(node.text[0]) <= 0x206f)))
+        is_drop_cap = (cssdict.get('float', None) == 'left' and 'font-size' in cssdict and len(node) == 0 and node.text and (
+            len(node.text) == 1 or (len(node.text) == 2 and 0x2000 <= ord(node.text[0]) <= 0x206f)))
         # Detect drop caps generated by the docx input plugin
-        if (node.tag and node.tag.endswith('}p') and len(node) == 0 and node.text and len(node.text.strip()) == 1 and
-            not node.tail and 'line-height' in cssdict and 'font-size' in cssdict):
+        if node.tag and node.tag.endswith('}p') and len(node) == 0 and node.text and len(node.text.strip()) == 1 and \
+                not node.tail and 'line-height' in cssdict and 'font-size' in cssdict:
             dp = node.getparent()
             if dp.tag and dp.tag.endswith('}div') and len(dp) == 1 and not dp.text:
                 if stylizer.style(dp).cssdict().get('float', None) == 'left':
@@ -473,8 +493,8 @@ class CSSFlattener(object):
         if cssdict:
             for x in self.filter_css:
                 popval = cssdict.pop(x, None)
-                if (self.body_font_family and popval and x == 'font-family' and
-                    popval.partition(',')[0][1:-1] == self.body_font_family.partition(',')[0][1:-1]):
+                if self.body_font_family and popval and x == 'font-family' \
+                    and popval.partition(',')[0][1:-1] == self.body_font_family.partition(',')[0][1:-1]:
                     cssdict[x] = popval
 
         if cssdict:
@@ -499,8 +519,7 @@ class CSSFlattener(object):
             lineh = self.lineh / psize
             cssdict['line-height'] = "%0.5fem" % lineh
 
-        if (self.context.remove_paragraph_spacing or
-                self.context.insert_blank_line) and tag in ('p', 'div'):
+        if (self.context.remove_paragraph_spacing or self.context.insert_blank_line) and tag in ('p', 'div'):
             if item_id != 'calibre_jacket' or self.context.output_profile.name == 'Kindle':
                 for prop in ('margin', 'padding', 'border'):
                     for edge in ('top', 'bottom'):
@@ -510,8 +529,7 @@ class CSSFlattener(object):
                     '%fem'%self.context.insert_blank_line_size
             indent_size = self.context.remove_paragraph_spacing_indent_size
             keep_indents = indent_size < 0.0
-            if (self.context.remove_paragraph_spacing and not keep_indents and
-                cssdict.get('text-align', None) not in ('center', 'right')):
+            if (self.context.remove_paragraph_spacing and not keep_indents and cssdict.get('text-align', None) not in ('center', 'right')):
                 cssdict['text-indent'] =  "%1.1fem" % indent_size
 
         pseudo_classes = style.pseudo_classes(self.filter_css)
@@ -597,7 +615,7 @@ class CSSFlattener(object):
             if item.media_type in OEB_STYLES:
                 manifest.remove(item)
         id, href = manifest.generate('css', 'stylesheet.css')
-        sheet = cssutils.parseString(css, validate=False)
+        sheet = css_parser.parseString(css, validate=False)
         if self.transform_css_rules:
             from calibre.ebooks.css_transform_rules import transform_sheet
             transform_sheet(self.transform_css_rules, sheet)
@@ -618,8 +636,7 @@ class CSSFlattener(object):
             items = sorted(stylizer.page_rule.items())
             css = ';\n'.join("%s: %s" % (key, val) for key, val in items)
             css = ('@page {\n%s\n}\n'%css) if items else ''
-            rules = [r.cssText for r in stylizer.font_face_rules +
-                    self.embed_font_rules]
+            rules = [r.cssText for r in stylizer.font_face_rules + self.embed_font_rules]
             raw = '\n\n'.join(rules)
             css += '\n\n' + raw
             global_css[css].append(item)
@@ -630,7 +647,7 @@ class CSSFlattener(object):
             href = None
             if css.strip():
                 id_, href = manifest.generate('page_css', 'page_styles.css')
-                sheet = cssutils.parseString(css, validate=False)
+                sheet = css_parser.parseString(css, validate=False)
                 if self.transform_css_rules:
                     from calibre.ebooks.css_transform_rules import transform_sheet
                     transform_sheet(self.transform_css_rules, sheet)
